<?php

/**
 * execute the auth process
 * - verification of the signature
 *
 * @param string $address
 * @param string $params
 *
 * @return array
 */
QUI::$Ajax->registerFunction(
    'package_quiqqer_authwalletconnect_ajax_auth',
    function ($address, $signature, $host) {
        if (empty($address) || empty($signature) || empty($host)) {
            return false;
        }

        $result = QUI::getDataBase()->fetch([
            'from'  => QUI\WalletConnect\WalletConnect::table(),
            'where' => [
                'address' => $address
            ],
            'limit' => 1
        ]);

        if (!isset($result[0])) {
            throw new Exception('Permission denied');
        }

        $message = QUI::getLocale()->get('quiqqer/authwalletconnect', 'signing.message', [
            'host'  => $host,
            'nonce' => $result[0]['nonce']
        ]);

        if (!\is_string($message)) {
            throw new Exception('Permission denied');
        }

        try {
            $WalletConnectAuth = new QUI\Auth\WalletConnect\Auth();

            $WalletConnectAuth->auth([
                'message'   => $message,
                'signature' => $signature,
                'address'   => $address
            ]);

            $WalletConnectAuth->setSession($address);
            $User = QUI::getUserBySession();

            return [
                'userId'   => $User->getId(),
                'uniqueId' => $User->getUniqueId(),
                'username' => $User->getUsername()
            ];
        } catch (QUI\Auth\WalletConnect\Exception $Exception) {
            return false;
        }
    },
    ['address', 'signature', 'host']
);
