/**
 * @module package/quiqqer/authwalletconnect/bin/controls/Login
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/authwalletconnect/bin/controls/Login', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Confirm',

    'Ajax',
    'Locale',

    'css!package/quiqqer/authwalletconnect/bin/controls/Login.css'

], function (QUI, QUIControl, QUILoader, QUIConfirm, QUIAjax, QUILocale) {
    "use strict";

    const lg = 'quiqqer/authwalletconnect';
    let utils;

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/authwalletconnect/bin/controls/Login',

        Binds: [
            '$onImport',
            'click'
        ],

        options: {
            uid: false  // QUIQQER User ID
        },

        initialize: function (options) {
            this.parent(options);

            this.addEvents({
                onImport: this.$onImport
            });

            this.$WalletConnect = null;

            this.Loader = new QUILoader();
            this.$nonce = null;
        },

        /**
         * @return {HTMLDivElement}
         */
        create: function () {
            this.$Elm = new Element('div', {
                'class': 'quiqqer-auth-walletconnect-login',
                'html' : '<div class="quiqqer-auth-walletconnect-login-info"></div>' +
                    '<img class="quiqqer-auth-walletconnect-login-icon" ' +
                    '   alt="Register now with walletconnect"' +
                    '   title="Register now with walletconnect"' +
                    '   src="' + window.URL_OPT_DIR + 'quiqqer/authwalletconnect/bin/WalletConnectLogo.png" ' +
                    '/>' +
                    '<div class="quiqqer-auth-walletconnect-login-btns">' +
                    '   <span>Register now with walletconnect</span>' +
                    '</div>'
            });

            this.$Elm.addEvent('click', this.click);
            this.Loader.inject(this.$Elm);

            return this.$Elm;
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            this.$Input      = this.getElm();
            this.$Input.type = 'hidden';
            this.$Form       = this.$Input.getParent('form');
            this.$autoLogin  = this.$Input.get('data-autologin') === "1";

            this.$Form.addClass('quiqqer-walletconnect-login-button');

            this.$Form.getElement('.quiqqer-walletconnect-login').set({
                'data-quiid': this.getId(),
                'data-qui'  : this.getType()
            });

            // node loaded
            var Parent       = this.getElm().getParent('.quiqqer-walletconnect-login-button');
            this.$NodeLoader = Parent.getElement('.quiqqer-fu-login-social-entry-loader');

            this.create().inject(this.$Input, 'after');
        },

        /**
         * login click
         */
        click: function () {
            let self = this;

            this.$showLoader();

            this.getWalletConnect().then(function () {
                return self.signing();
            }).then(function (signature) {
                const address = self.$WalletConnect.accounts[0];
                const host    = window.location.host;

                return self.auth(address, signature, host);
            }).then(function (authenticated) {
                if (authenticated) {
                    self.$hideLoader();
                    window.location.reload();
                    return;
                }

                self.$hideLoader();
            }).catch(function (err) {
                if (err.toString().indexOf('User closed modal') !== -1) {
                    self.$hideLoader();
                    return;
                }

                console.error(err);

                if (typeof err.getMessage === 'function') {
                    QUI.getMessageHandler().then(function (MH) {
                        MH.addError(err.getMessage());
                    });
                }

                self.$hideLoader();
            });
        },

        /**
         * Return the WalletConnect Object
         *
         * @return {Promise}
         */
        getWalletConnect: function () {
            var self = this;

            return new Promise(function (resolve, reject) {
                if (self.$WalletConnect) {
                    resolve(self.$WalletConnect);
                    return;
                }

                window.localStorage.removeItem('walletconnect');

                var walletConnectDir = URL_VAR_DIR + 'package/quiqqer/authwalletconnect/bin/node_modules/@walletconnect/';

                require.config({
                    paths: {
                        'WalletConnect'           : walletConnectDir + 'client/dist/umd/index.min',
                        'WalletConnectQRCodeModal': walletConnectDir + 'qrcode-modal/dist/umd/index.min',
                        'WalletConnectUtils'      : walletConnectDir + 'utils/dist/umd/index.min',
                        'WalletConnectProvider'   : walletConnectDir + 'web3-provider/dist/umd/index.min'
                    }
                });

                require([
                    'WalletConnect',
                    'WalletConnectQRCodeModal',
                    'WalletConnectProvider'
                ], function (WalletConnect, QRCodeModal, WalletConnectProvider) {
                    self.$loadUtils().then(function () {

                        self.$WalletConnect = new WalletConnectProvider.default({
                            infuraId: "27e484dcd9e3efcfd25a83a78777cdf1",
                        });

                        var onError = function (err) {
                            self.$WalletConnect = null;
                            reject(err);
                        };

                        var onSuccess = function () {
                            resolve(self.$WalletConnect);
                        };

                        // var peerId = self.$WalletConnect.wc.peerId;
                        if (self.$WalletConnect.isWalletConnect) {
                            self.$WalletConnect.enable().then(onSuccess).catch(onError);

                            return;
                        }

                        self.$WalletConnect.enable().then(onSuccess).catch(onError);

                    });
                });
            });
        },

        /**
         * Signing message
         *
         * @return {Promise}
         */
        signing: function () {
            const self = this;

            let web3;

            return this.getWalletConnect().then(function () {
                return self.getWeb3();
            }).then(function (w3) {
                web3 = w3;

                return self.$getNonce();
            }).then(function () {
                let message = QUILocale.get(lg, 'signing.message', {
                    nonce: self.$nonce,
                    host : window.location.host
                });

                return new Promise(function (resolve, reject) {
                    web3.eth.personal.sign(
                        utils.convertUtf8ToHex(message),
                        self.$WalletConnect.accounts[0],
                        function (err, signature) {
                            if (err) {
                                return reject(err);
                            }

                            return resolve(signature);
                        }
                    )
                });
            });
        },

        /**
         * get web3 client
         *
         * @return {Promise}
         */
        getWeb3: function () {
            const self = this;

            return new Promise(function (resolve) {
                require([URL_OPT_DIR + 'quiqqer/authmetamask/bin/web3.min.js'], function (Web3) {
                    resolve(new Web3(self.$WalletConnect));
                });
            });
        },

        /**
         * Return the connected wallet address
         *
         * @return {Promise<unknown>}
         */
        $getAddress: function () {
            return this.getWalletConnect().then(function (WalletConnect) {
                return WalletConnect.accounts[0];
            });
        },

        /**
         * @return {Promise}
         */
        $getNonce: function () {
            if (this.$nonce) {
                return Promise.resolve(this.$nonce);
            }

            const self = this;

            return this.$getAddress().then(function (address) {
                const checksum = utils.toChecksumAddress(address);

                console.log('$getNonce', checksum);

                return new Promise(function (resolve) {
                    QUIAjax.get('package_quiqqer_authwalletconnect_ajax_login', function (nonce) {
                        self.$nonce = nonce;
                        resolve(nonce);
                    }, {
                        'package': 'quiqqer/authwalletconnect',
                        address  : address,
                        onError  : resolve
                    });
                });
            });
        },

        /**
         *
         * @param address
         * @param signature
         * @param host
         * @return {*}
         */
        auth: function (address, signature, host) {
            const checksum = utils.toChecksumAddress(address);

            console.log('AUTH', checksum);

            return new Promise(function (resolve, reject) {
                QUIAjax.post('package_quiqqer_authwalletconnect_ajax_auth', resolve, {
                    'package': 'quiqqer/authwalletconnect',
                    onError  : reject,
                    address  : address,
                    signature: signature,
                    host     : host,
                });
            });
        },

        /**
         * show loaders
         */
        $showLoader: function () {
            this.Loader.show();
            this.$NodeLoader.setStyles({
                display: 'inline-block',
                opacity: 0.5,
            });
        },

        /**
         * hide loaders
         */
        $hideLoader: function () {
            this.Loader.hide();
            this.$NodeLoader.setStyles({
                display: null,
                opacity: null
            });
        },

        /**
         * @return {Promise}
         */
        $loadUtils: function () {
            if (utils) {
                return Promise.resolve();
            }

            return new Promise(function (resolve) {
                require(['WalletConnectUtils'], function (u) {
                    utils = u;
                    resolve();
                });
            });
        }
    });
});

